#pragma NOIV               // Do not generate interrupt vectors
//-----------------------------------------------------------------------------
//   File:      bulksrc.c
//   Contents:   FIFO Example implementation
//
//   Copyright (c) 2010 Manuel Di Cerbo, FHNW
//   Copyright (c) 2009 Matthias Meier, FHNW
//   Copyright (c) 2000 Cypress Semiconductor All rights reserved
//-----------------------------------------------------------------------------
#include "fx2.h"
#include "fx2regs.h"
#include "syncdly.h"            // SYNCDELAY macro


//LEDs an PORT C, Bit 4..7
#define LEDS			IOC
#define LEDS_OE			OEC
#define	LEDS_MASK 		0xf0

#define I2C_ADC_ADDR    0x00	//7 Bit Address of CPLD I2C


extern BOOL GotSUD;             // Received setup data flag
extern BOOL Sleep;
extern BOOL Rwuen;
extern BOOL Selfpwr;

BYTE Configuration;             // Current configuration
BYTE AlternateSetting;          // Alternate settings

WORD packetSize;


//-----------------------------------------------------------------------------
// Task Dispatcher hooks
//   The following hooks are called by the task dispatcher.
//-----------------------------------------------------------------------------

void TD_Init(void)              // Called once at startup
{
  int i = 0;

  LEDS_OE = LEDS_MASK;        //LED PORT-Bits auf Output
  LEDS = 0x00 & LEDS_MASK;           //LEDs off (da aktive low)
  // set the CPU clock to 48MHz
  CPUCS = ((CPUCS & ~bmCLKSPD) | bmCLKSPD1) ;
  SYNCDELAY;

  // set the slave FIFO interface to 48MHz, default is 0x80; int. clocked
  IFCONFIG = 0xE3;//INTERNAL 48 MHZ IFCLK
//  IFCONFIG = 0x43;//EXTERNAL IFCLK

  //normal mode
  //IFCONFIG = 0xE0;
  SYNCDELAY;
	
  REVCTL = 0x03;   //Revision Control, enable operations
  SYNCDELAY;

  //invalidate unused eps
  EP4CFG = 0x00;
  EP8CFG = 0x00;

  //reset routine
  FIFORESET = 0x80; //nak incomming requests
  SYNCDELAY;
  FIFORESET = 0x02; //reset ep2
  SYNCDELAY;
  FIFORESET = 0x04; //reset ep2
  SYNCDELAY;
  FIFORESET = 0x06; //reset ep2
  SYNCDELAY;
  FIFORESET = 0x08; //reset ep2
  SYNCDELAY;
  FIFORESET = 0x00; //resume
  SYNCDELAY;

  //EP2 FIFO Config
  EP2FIFOCFG = 0x0D; //AUTOIN, ZEROLENIN, WORDWIDE 1
  SYNCDELAY;

  //EP6 same
  EP6FIFOCFG = 0x0D; //AUTOIN, ZEROLENIN, WORDWIDE 1
  SYNCDELAY; 

//  EP2 Config Quad Bulk IN 512
  EP2CFG = 0xE0;
  SYNCDELAY;

  //EP6 same cfg
  EP6CFG = 0xE0;
  SYNCDELAY;

  //EP1 for command messages
  EP1INCFG = 0xA0;
  SYNCDELAY;

  //EP1 OUT for incomming messages
  EP1OUTCFG = 0xA0;
  SYNCDELAY;


  EP2AUTOINLENH = 0x02; // Auto-commit 512-byte packets
  SYNCDELAY;
  EP2AUTOINLENL = 0x00;
  SYNCDELAY;

  EP6AUTOINLENH = 0x02; // same as ep2
  SYNCDELAY;
  EP6AUTOINLENL = 0x00;
  SYNCDELAY;

  //set polarity of FIFO Control Pins 
  FIFOPINPOLAR = 0x38; //inv PKTEND, SLOE and SLRD
  SYNCDELAY;


  for(i = 0; i < 512; i++){
  	EP2FIFOBUF[i] = 0;
	SYNCDELAY;
  }
  for(i = 0; i < 512; i++){
  	EP2FIFOBUF[i] = 0;
	SYNCDELAY;
  }
  for(i = 0; i < 512; i++){
  	EP2FIFOBUF[i] = 0;
	SYNCDELAY;
  }
  for(i = 0; i < 512; i++){
  	EP2FIFOBUF[i] = 0;
	SYNCDELAY;
  }

  for(i = 0; i < 64; i++){
  	EP1INBUF[i] = i;
  }

  EP1INBC = 64;
  SYNCDELAY;

  EP1OUTBC = 0;//arm out p144 ref man
  SYNCDELAY;

  SYNCDELAY;
  EP2BCL = 0x00;
  SYNCDELAY;
  
  EP6BCH = 0x02;
  SYNCDELAY;
  EP6BCL = 0x00;
  SYNCDELAY;
  

  EZUSB_InitI2C();//init i2c
 }

void TD_Poll(void)              // Called repeatedly while the device is idle
{
	int i = 0;
	//Handle EP1
	//the rest is done by the slave fifo
	if(!(EP1INCS & bmBIT1)){
		for(i = 0; i < 64; i++){
			EP1INBUF[i] = i*4;
			SYNCDELAY;
		}
		EP1INBC = 64;
		SYNCDELAY;
	}

	//incomming
	if(!(EP1OUTCS & bmBIT1)){
		EZUSB_WriteI2C(I2C_ADC_ADDR, 1, EP1OUTBUF);
		SYNCDELAY;
		EP1OUTBC = 0;//re-arm out
		SYNCDELAY;
	}
}

BOOL TD_Suspend(void)          // Called before the device goes into suspend mode
{
   return(TRUE);
}

BOOL TD_Resume(void)          // Called after the device resumes
{
   return(TRUE);
}

//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------

BOOL DR_GetDescriptor(void)
{
   return(TRUE);
}

BOOL DR_SetConfiguration(void)   // Called when a Set Configuration command is received
{
   Configuration = SETUPDAT[2];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetConfiguration(void)   // Called when a Get Configuration command is received
{
   EP0BUF[0] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_SetInterface(void)       // Called when a Set Interface command is received
{
   AlternateSetting = SETUPDAT[2];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetInterface(void)       // Called when a Set Interface command is received
{
   EP0BUF[0] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_GetStatus(void)
{
   return(TRUE);
}

BOOL DR_ClearFeature(void)
{
   return(TRUE);
}

BOOL DR_SetFeature(void)
{
   return(TRUE);
}

BOOL DR_VendorCmnd(void)
{
   return(TRUE);
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav(void) interrupt 0
{
   GotSUD = TRUE;            // Set flag
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUDAV;         // Clear SUDAV IRQ
}

// Setup Token Interrupt Handler
void ISR_Sutok(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUTOK;         // Clear SUTOK IRQ
}

void ISR_Sof(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSOF;            // Clear SOF IRQ
}

void ISR_Ures(void) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      pOtherConfigDscr = pFullSpeedConfigDscr;
      packetSize = 512;

   }
   else
   {
      pConfigDscr = pFullSpeedConfigDscr;
      pOtherConfigDscr = pHighSpeedConfigDscr;
      packetSize = 64;
   }
   
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ
}

void ISR_Susp(void) interrupt 0
{
   Sleep = TRUE;
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUSP;
}

void ISR_Highspeed(void) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      pOtherConfigDscr = pFullSpeedConfigDscr;
      packetSize = 512;

   }
   else
   {
      pConfigDscr = pFullSpeedConfigDscr;
      pOtherConfigDscr = pHighSpeedConfigDscr;
      packetSize = 64;
   }

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmHSGRANT;
}
void ISR_Ep0ack(void) interrupt 0
{
}
void ISR_Stub(void) interrupt 0
{
}
void ISR_Ep0in(void) interrupt 0
{
}
void ISR_Ep0out(void) interrupt 0
{
}
void ISR_Ep1in(void) interrupt 0
{
}
void ISR_Ep1out(void) interrupt 0
{
}
void ISR_Ep2inout(void) interrupt 0
{
}
void ISR_Ep4inout(void) interrupt 0
{
}
void ISR_Ep6inout(void) interrupt 0
{
}
void ISR_Ep8inout(void) interrupt 0
{
}
void ISR_Ibn(void) interrupt 0
{
}
void ISR_Ep0pingnak(void) interrupt 0
{
}
void ISR_Ep1pingnak(void) interrupt 0
{
}
void ISR_Ep2pingnak(void) interrupt 0
{
}
void ISR_Ep4pingnak(void) interrupt 0
{
}
void ISR_Ep6pingnak(void) interrupt 0
{
}
void ISR_Ep8pingnak(void) interrupt 0
{
}
void ISR_Errorlimit(void) interrupt 0
{
}
void ISR_Ep2piderror(void) interrupt 0
{
}
void ISR_Ep4piderror(void) interrupt 0
{
}
void ISR_Ep6piderror(void) interrupt 0
{
}
void ISR_Ep8piderror(void) interrupt 0
{
}
void ISR_Ep2pflag(void) interrupt 0
{
}
void ISR_Ep4pflag(void) interrupt 0
{
}
void ISR_Ep6pflag(void) interrupt 0
{
}
void ISR_Ep8pflag(void) interrupt 0
{
}
void ISR_Ep2eflag(void) interrupt 0
{
}
void ISR_Ep4eflag(void) interrupt 0
{
}
void ISR_Ep6eflag(void) interrupt 0
{
}
void ISR_Ep8eflag(void) interrupt 0
{
}
void ISR_Ep2fflag(void) interrupt 0
{

}
void ISR_Ep4fflag(void) interrupt 0
{
}
void ISR_Ep6fflag(void) interrupt 0
{
}
void ISR_Ep8fflag(void) interrupt 0
{
}
void ISR_GpifComplete(void) interrupt 0
{
}
void ISR_GpifWaveform(void) interrupt 0
{
}
